#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

import pygst
pygst.require("0.10")
import gst
import pygtk
# mandatory for installation with both pygtk1 and pygtk2
pygtk.require("2.0")
import gtk
import gobject
import gtk.glade
import os, sys, glob, time, glob, gzip

import mutagen
from mutagen import  flac,id3,oggvorbis
from mutagen.flac import FLAC
from mutagen.id3 import ID3
from mutagen.oggvorbis import OggVorbis

import name
from name import name

import audiofilestags

# This variable contains the unique instance to the work window
TheAudiofiles = None

def LaunchAudiofiles (f_infos,f_word_dic,f_widget,f_application):
    global TheAudiofiles
    if TheAudiofiles is None :
        TheAudiofiles = Audiofiles(f_infos,f_word_dic,f_widget,f_application)
        TheAudiofiles.main()
    else:
        print "You can only launch one audiofile box at a time"
    
        # option 1: dialog box to complain that window is already open
        # option 2: reconfigure existing window and bring it to top
        # option 3: destroy old window then create a new one


class Audiofiles:
    """
    this class is responsible of converting the audio flac files
    to flac (true ones ...), ogg, mp3 and wav files files
    it has also to build the local repository if necessary
    and to generate all the files needed
    """

    def __init__(self,f_infos,f_word_dic,f_widget,f_application):
        print "launching the audiofiles class"
        self.infos = f_infos
        # DEBUG:
        print self.infos
        self.word_dic = f_word_dic
        self.generate_button = f_widget
        l_dir = os.path.dirname(__file__)
        l_file_name = os.path.join (l_dir, "audiofiles.glade")
        # DEBUG:
        #print l_file_name
        self.xml_glade = gtk.glade.XML(fname=l_file_name,domain=f_application)
        self.dialog1 = self.xml_glade.get_widget("dialog1")
        dic={"on_dialog1_delete_event"            : self.on_window1_delete,
             "on_dialog1_destroy"                 : self.on_window1_destroy,
             "on_button1_clicked"                 : self.on_button1_clicked,
             "on_button2_clicked"                 : self.on_button2_clicked}
        self.xml_glade.signal_autoconnect(dic)
    
    def on_button1_clicked(self,widget,data=None):
        global TheAudiofiles
        TheAudiofiles = None
        self.generate_button.set_sensitive(True)
        self.dialog1.destroy()
        
    def on_button2_clicked(self,widget,data=None):
        pass
    
    
    def on_window1_delete(self,widget, data=None):
        return False
    
    def on_window1_destroy(self,widget,data=None):
        global TheAudiofiles
        TheAudiofiles = None
        self.generate_button.set_sensitive(True)  
        
    def fill_header_txt(self,f_file, f_content):
        # fill the txt headers
        f_file.write("\Swac_Index_Tags\n\n[GLOBAL]" + "\n")
        l = f_content.split("\n")
        # DEBUG:
        #print "l", l
        for ll in l:
            if ll.strip("\n\f\r") != "":
                f_file.write(ll + "\n")
        f_file.write("\n")
        
    def fill_header_xml(self,f_file,f_content):
        # fill the xml headers  
        f_file.write('<?xml version="1.0" encoding="utf8"?>\n' + \
                    '<index name="Swac_Index_Tags">\n\n' + \
                    '<item name="GLOBAL">')
        l = f_content.split("\n")
        # DEBUG:
        #print "l", l
        for ll in l:
            #print "ll",ll
            if ll.strip("\n\f\r") != "":
                lll = ll.split("=")
                #print "lll",lll
                f_file.write('  <field name="' + lll[0] +'">' + lll[1] + '</field>' + "\n")
        f_file.write("</item>" + "\n\n")   
        
    def populate_body_txt(self,f_file,f_index,f_naming_class,f_ext):
        # fill the body of text index files
        f_index.seek(0)        
        for l in f_index:
            l = l.strip("\n\f\r")
            if l == "":
                continue
            ll = l.split("::")
            # DEBUG:
            #print "ll0= ", ll[0]
            #print "ll1= ", ll[1]
            l_dest = f_naming_class.get(ll[1],self.infos.list_langage,self.infos.locutor_langage_learning_country) + f_ext
            # DEBUG:
            #print "l_dest",l_dest
            f_file.write("[" + l_dest + "]" + "\n" + "TITLE=" + ll[2] + "\n" + "SWAC_TEXT=" + ll[1] + "\n" + "SWAC_TECH_DATE=" + ll[3] + "\n\n")

        
        
    def populate_body_xml(self,f_file,f_index,f_naming_class,f_ext):
        # fill the body of xml index files
        # DEBUG:
        #print "start body xml"
        f_index.seek(0)     
        for l in f_index:
            l = l.strip("\n\f\r")
            if l == "":
                continue
            ll = l.split("::")
            # DEBUG:
            #print "ll0= ", ll[0]
            #print "ll1= ", ll[1]
            l_dest = f_naming_class.get(ll[1],self.infos.list_langage,self.infos.locutor_langage_learning_country) + f_ext
            # DEBUG:
            #print "l_dest",l_dest
            f_file.write('<item name="' + l_dest + '">' + "\n" + \
                            '  <field name="TITLE">' + ll[2] + '</field>' + "\n" + \
                            '  <field name="SWAC_TEXT">' + ll[1] + '</field>' + "\n" + \
                            '  <field name="SWAC_ALPHAIDX">' + ll[1] + '</field>' + "\n" + \
                            '  <field name="SWAC_TECH_DATE">'+ ll[3] + '</field>' + "\n" + \
                            '</item>' + "\n\n") 
        f_file.write("\n" + "</index>" + "\n")
        #print "end body xml"
                  
    def dialog1_main(self):
        # the naming sheme:
        if self.infos.name_style == 0:
            my_naming_class =  name.YazikName()
        elif self.infos.name_style == 1:
            my_naming_class =  name.WikiDictionnaryName()
        elif self.infos.name_style == 2:
            my_naming_class =  name.ShtookaName()   
        else:
            my_naming_class = name.CRC32Name() 
        # building the dir structure if needed
        # the data directory
        # the liste directory
        # the ogg flac mp3 directories
        self.label.set_text("Building the directory structure")
        self.pending()
        # the home and .yazik dirs
        self.home_dir = os.path.expanduser("~")
        self.yazik_dir = os.path.join(self.home_dir,".yazik/")
        self.audio_dir = os.path.join(self.yazik_dir,"audio")
        self.audio_tmp_dir = os.path.join(self.yazik_dir,"audio/tmp")
        # the data dir
        self.data_dir = os.path.join(self.yazik_dir,"data")
        if  not os.path.lexists(self.data_dir):
            self.progress_bar.set_text(self.data_dir)
            self.progress_bar.set_fraction(0.2)
            self.pending()
            try:
                os.mkdir(self.data_dir)
            except:
                # TODO: better managment
                print "Error: cannot create the home data dir"
        # the list dir
        self.list_dir = os.path.join(self.data_dir,self.infos.list_name)
        if  not os.path.lexists(self.list_dir):
            self.progress_bar.set_text(self.list_dir)
            self.progress_bar.set_fraction(0.4)
            self.pending()
            try:
                os.mkdir(self.list_dir)
            except:
                print "Error: cannot create the home list dir"
        # the mp3 ogg flac dirs
        if self.infos.mp3:
            # DEBUG:
            #print "self.infos.mp3",self.infos.mp3, "creating the dir"
            #print type(self.infos.mp3)
            self.mp3_dir = os.path.join(self.list_dir,"mp3")
            if  not os.path.lexists(self.mp3_dir):
                self.progress_bar.set_text(self.mp3_dir)
                self.progress_bar.set_fraction(0.6)
                self.pending()
                try:
                    os.mkdir(self.mp3_dir)
                except:
                    print "Error: cannot create the list mp3 dir"
        if self.infos.ogg:
            self.ogg_dir = os.path.join(self.list_dir,"ogg")
            if  not os.path.lexists(self.ogg_dir):
                self.progress_bar.set_text(self.ogg_dir)
                self.progress_bar.set_fraction(0.8)
                self.pending()
                try:
                    os.mkdir(self.ogg_dir)
                except:
                    print "Error: cannot create the list ogg dir"
        if self.infos.flac:
            self.flac_dir = os.path.join(self.list_dir,"flac")
            if  not os.path.lexists(self.flac_dir):
                self.progress_bar.set_text(self.flac_dir)
                self.progress_bar.set_fraction(1.0)
                self.pending()
                try:
                    os.mkdir(self.flac_dir)
                except:
                    print "Error: cannot create the list flac dir"
        self.progress_bar.set_text("")
        self.progress_bar.set_fraction(0.0)
        self.label.set_text("Directory structure created")
        self.pending()
        # creating the auxiliary files as asked
        self.label.set_text("Creating the auxiliary files")
        self.pending()
        try:
            file1 = open(os.path.join(self.audio_dir,self.infos.list_name +".tot.txt"),"w")
            file1.write("\n")
            file1.write("SWAC_SPEAK_NAME=" + self.infos.locutor_name + "\n")
            file1.write("COPYRIGHT=" + self.infos.locutor_copyright + "\n")
            #file1.write("DESCRIPTION=" +
            file1.write("GENRE=Speech" +  "\n")
            file1.write("LICENSE=" + self.infos.locutor_license + "\n")
            file1.write("SWAC_LANG=" + self.infos.list_langage + "\n")
            file1.write("SWAC_COLL_ORG=Yazik Project" + "\n")
            file1.write("SWAC_COLL_ORG_URL=http://www.yojik.eu/" + "\n")
            file1.write("SWAC_COLL_URL=http://packs.yojik.eu/"+ self.infos.list_name + "\n")
            file1.write("SWAC_TECH_SOFT=Yazikrecorder" + "\n")
            if self.infos.locutor_gender == 0:
                l_gender = "M"
            else:
                l_gender = "F"
            file1.write("SWAC_SPEAK_GENDER=" + l_gender + "\n")
            file1.write("SWAC_SPEAK_BIRTH_YEAR=" + self.infos.locutor_year_of_birth + "\n")
            file1.write("SWAC_SPEAK_LANG=" + self.infos.locutor_native_langage + "\n")
            file1.write("SWAC_SPEAK_LANG_COUNTRY=" + self.infos.locutor_langage_learning_country + "\n" )
            file1.write("SWAC_SPEAK_LANG_REGION=" + self.infos.locutor_linguistic_region + "\n" )
            file1.write("SWAC_SPEAK_LIV_TOWN=" + self.infos.locutor_living_town + "\n")
            file1.write("SWAC_SPEAK_LIV_COUNTRY=" + self.infos.locutor_living_country + "\n")
            file1.write("SWAC_COLL_LICENSE=" + self.infos.locutor_license + "\n")
            file1.write("SWAC_COLL_COPYRIGHT=" + self.infos.locutor_copyright + "\n")
            file1.write("DESCRIPTION=" + self.infos.short_collection_description + "\n")
            file1.write("SWAC_PRON_SPEED=" + str(self.infos.speed + 1) + "\n")
            file1.write("SWAC_COLL_NAME=" + self.infos.short_collection_description + "\n")
            file1.write("SWAC_COLL_SECTION=" + self.infos.collection_section + "\n")
            file1.write("SWAC_COLL_DESC=" + self.infos.collection_description + "\n")
            file1.write("SWAC_COLL_AUTHORS=" + self.infos.collection_authors + "\n")
            file1.write("SWAC_TECH_QLT=" + str(self.infos.quality + 1) + "\n")
            if self.infos.name_style == 0:
                a = "Yazik"
            elif self.infos.name_style == 1:
                a = "Wikidictionnary"
            elif self.infos.name_style == 2:
                a = "Shtooka"
            else:
                a = "Other"
            file1.write("YAZIK_NAME_STYLE=" + a + "\n\n")
            file1.close()            
        except:
            print "not possible to create the tot file"
        # the index file
        try:
            file2 = open(os.path.join(self.audio_dir,self.infos.list_name +".index.txt"),"w")
            file2.write("\n")
            for k in self.word_dic.keys():
                if self.word_dic[k][1]== 1:
                    # crc32 + text + tech_date + Tile
                    l_source = os.path.join(self.audio_tmp_dir,k + ".flac")
                    l_date = audiofilestags.get_date(l_source)
                    l_dic = audiofilestags.build_dic(self.infos, self.word_dic[k][0], l_date)
                    file2.write(k + "::" + self.word_dic[k][0] + "::" + l_dic["SWAC_COLL_NAME"] + " - " + l_dic["SWAC_TEXT"] + "::" + l_date + "\n")
            file2.close()
        except:
            print "not possible to create the index file"
        self.label.set_text("Generating and tagging the audio files")
        self.pending()
        length_word_dic = len(self.word_dic)        
        if self.infos.mp3:            
            i = 0
            ii = float(1.0 / length_word_dic)
            for k in self.word_dic.keys():
                if self.word_dic[k][1]== 1:
                    i = i + ii
                    l_source = os.path.join(self.audio_tmp_dir,k + ".flac")
                    #print "source", l_source
                    self.progress_bar.set_text(l_source)
                    self.progress_bar.set_fraction(i)
                    self.pending()
                    l_dest = my_naming_class.get(self.word_dic[k][0],self.infos.list_langage,self.infos.locutor_langage_learning_country) + ".mp3"
                    l_dest = os.path.join(self.mp3_dir,l_dest)
                    #print "destination", l_dest
                    encoding_string = 'filesrc location=%(source)s !  %(decoder)s ! %(encoder)s ! filesink location=%(dest)s' \
                                       % {"source": l_source , "decoder": "flacdec", "encoder": "lame","dest": l_dest}
                    #print "encoding string", encoding_string
                    command = "gst-launch-0.10 " + encoding_string
                    #print "command",command
                    os.system(command)
                    l_date = audiofilestags.get_date(l_source)
                    l_dic = audiofilestags.build_dic(self.infos, self.word_dic[k][0], l_date)
                    audiofilestags.tag_mp3_file(l_dest, l_dic)
                    
        if self.infos.ogg:
            i = 0
            ii = float(1.0 / length_word_dic)
            for k in self.word_dic.keys():
                if self.word_dic[k][1]== 1:
                    i = i + ii
                    l_source = os.path.join(self.audio_tmp_dir,k + ".flac")
                    #print "source", l_source
                    self.progress_bar.set_text(l_source)
                    self.progress_bar.set_fraction(i)
                    self.pending()   
                    l_dest = my_naming_class.get(self.word_dic[k][0],self.infos.list_langage,self.infos.locutor_langage_learning_country) + ".ogg"
                    l_dest = os.path.join(self.ogg_dir,l_dest)
                    #print "destination", l_dest
                    encoding_string = 'filesrc location=%(source)s !  %(decoder)s ! audioconvert ! %(encoder)s ! filesink location=%(dest)s' \
                                       % {"source": l_source , "decoder": "flacdec", "encoder": " vorbisenc ! oggmux","dest": l_dest}
                    #print "encoding string", encoding_string
                    command = "gst-launch-0.10 " + encoding_string
                    #print "command",command
                    os.system(command)
                    l_date = audiofilestags.get_date(l_source)
                    l_dic = audiofilestags.build_dic(self.infos, self.word_dic[k][0], l_date)
                    audiofilestags.tag_ogg_file(l_dest, l_dic)
                    
        if self.infos.flac:
            i = 0
            ii = float(1.0 / length_word_dic)
            for k in self.word_dic.keys():
                if self.word_dic[k][1]== 1:
                    i = i + ii
                    l_source = os.path.join(self.audio_tmp_dir,k + ".flac")
                    #print "source", l_source
                    self.progress_bar.set_text(l_source)
                    self.progress_bar.set_fraction(i)
                    self.pending()
                    l_dest = my_naming_class.get(self.word_dic[k][0],self.infos.list_langage,self.infos.locutor_langage_learning_country) + ".flac"
                    l_dest = os.path.join(self.flac_dir,l_dest)
                    #print "destination", l_dest
                    encoding_string = 'filesrc location=%(source)s !  %(decoder)s ! %(encoder)s ! filesink location=%(dest)s' \
                                       % {"source": l_source , "decoder": "flacdec", "encoder": "flacenc","dest": l_dest}
                    #print "encoding string", encoding_string
                    command = "gst-launch-0.10 " + encoding_string
                    #print "command",command
                    os.system(command)
                    # tagging the file
                    l_date = audiofilestags.get_date(l_source)
                    l_dic = audiofilestags.build_dic(self.infos, self.word_dic[k][0], l_date)
                    # DEBUG: 
                    #print l_dic
                    audiofilestags.tag_flac_file(l_dest, l_dic) 
        self.progress_bar.set_text("")
        self.progress_bar.set_fraction(0.0)
        self.label.set_text("Audio files generated and tagged")
        self.pending()
        # generating the auxiliary files
        self.label.set_text("Generating the auxiliary files")
        try:
            # header part
            file1 = open(os.path.join(self.audio_dir,self.infos.list_name +".tot.txt"),"r")
            file1_content = file1.read()
            file1.close()
            file2 = open(os.path.join(self.audio_dir,self.infos.list_name +".index.txt"),"r")
            if self.infos.shtooka_txt:
                if self.infos.flac:
                    file3 = open(os.path.join(self.flac_dir,"index.tags.txt"),"w")
                    self.fill_header_txt(file3,file1_content)
                if self.infos.ogg:
                    file4 = open(os.path.join(self.ogg_dir,"index.tags.txt"),"w")
                    self.fill_header_txt(file4,file1_content)
                if self.infos.mp3:
                    file5 = open(os.path.join(self.mp3_dir,"index.tags.txt"),"w")
                    self.fill_header_txt(file5,file1_content)
            if self.infos.shtooka_xml:
                if self.infos.flac:
                    file6 = open(os.path.join(self.flac_dir,"index.tags.xml"),"w")
                    self.fill_header_xml(file6,file1_content)
                if self.infos.ogg:
                    file7 = open(os.path.join(self.ogg_dir,"index.tags.xml"),"w")
                    self.fill_header_xml(file7,file1_content)
                if self.infos.mp3:
                    file8 = open(os.path.join(self.mp3_dir,"index.tags.xml"),"w")
                    self.fill_header_xml(file8,file1_content)
            # body part 
            if self.infos.shtooka_txt:
                if self.infos.flac:
                    self.populate_body_txt(file3,file2,my_naming_class,".flac") 
                if self.infos.ogg:
                    self.populate_body_txt(file4,file2,my_naming_class,".ogg")
                if self.infos.mp3:
                    self.populate_body_txt(file5,file2,my_naming_class,".mp3")
            if self.infos.shtooka_xml:
                if self.infos.flac:
                    #print "populate xml flac body"
                    self.populate_body_xml(file6,file2,my_naming_class,".flac")
                if self.infos.ogg:
                    self.populate_body_xml(file7,file2,my_naming_class,".ogg")
                if self.infos.mp3:
                    self.populate_body_xml(file8,file2,my_naming_class,".mp3")
            if self.infos.shtooka_txt:
                if self.infos.flac: 
                    #print "closing file3"
                    file3.close()
                if self.infos.ogg: 
                    file4.close()
                if self.infos.mp3: 
                    file5.close()
            if self.infos.shtooka_xml:
                if self.infos.flac: 
                    file6.close()
                if self.infos.ogg: 
                    file7.close()
                if self.infos.mp3: 
                    file8.close() 
            #print "end text and xml index files generated"
            # compression
            if self.infos.shtooka_txt:
                if self.infos.flac:
                    file3 = open(os.path.join(self.flac_dir,"index.tags.txt"),"r")
                    content = file3.read()
                    #print " content read "
                    fileObj = gzip.GzipFile(os.path.join(self.flac_dir,"index.tags.txt.gz"), 'wb');
                    fileObj.write(content)
                    fileObj.close()
                    file3.close()
                if self.infos.ogg:
                    file4 = open(os.path.join(self.ogg_dir,"index.tags.txt"),"r") 
                    content = file4.read()
                    #print " content read "
                    fileObj = gzip.GzipFile(os.path.join(self.ogg_dir,"index.tags.txt.gz"), 'wb');
                    fileObj.write(content)
                    fileObj.close()
                    file4.close()
                if self.infos.mp3: 
                    file5 = open(os.path.join(self.mp3_dir,"index.tags.txt"),"r")
                    content = file5.read()
                    #print " content read "
                    fileObj = gzip.GzipFile(os.path.join(self.mp3_dir,"index.tags.txt.gz"), 'wb');
                    fileObj.write(content)
                    fileObj.close()
                    file5.close()
            if self.infos.shtooka_xml:
                if self.infos.flac: 
                    file6 = open(os.path.join(self.flac_dir,"index.tags.xml"),"r")
                    content = file6.read()
                    #print " content read "
                    fileObj = gzip.GzipFile(os.path.join(self.flac_dir,"index.tags.xml.gz"), 'wb');
                    fileObj.write(content)
                    fileObj.close()
                    file6.close()
                if self.infos.ogg: 
                    file7 = open(os.path.join(self.ogg_dir,"index.tags.xml"),"r")
                    content = file7.read()
                    #print " content read "
                    fileObj = gzip.GzipFile(os.path.join(self.ogg_dir,"index.tags.xml.gz"), 'wb');
                    fileObj.write(content)
                    fileObj.close()
                    file7.close()
                if self.infos.mp3: 
                    file8 = open(os.path.join(self.mp3_dir,"index.tags.xml"),"r")
                    content = file8.read()
                    #print " content read "
                    fileObj = gzip.GzipFile(os.path.join(self.mp3_dir,"index.tags.xml.gz"), 'wb');
                    fileObj.write(content)
                    fileObj.close()
                    file8.close()   
        except:
            print "Error: not possible to create Shtooka index files"
      
        self.pending()
        self.progress_bar.set_text("")
        self.progress_bar.set_fraction(0.0)
        self.label.set_text("Auxiliary files generated")
        # DEBUG:
        #print "flac",self.infos.flac,"mp3",self.infos.mp3,"ogg",self.infos.ogg
        # end of generation
        time.sleep(0.5)
        self.label.set_text("You may close this window\n as eveything is OK")
        
    def pending(self):
        while gtk.events_pending():
            gtk.main_iteration()
        
    def main(self):
        self.progress_bar = self.xml_glade.get_widget("progressbar1")
        self.label = self.xml_glade.get_widget("label1")
        self.button2 = self.xml_glade.get_widget("button2")
        self.button2.set_sensitive(False)
        while gtk.events_pending():
            gtk.main_iteration()
        #source_id = gobject.timeout_add(6000, self.my_splash.hide) # splash screen ...
        gobject.idle_add(self.dialog1_main)
        
        
        