#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

import os, stat, time

import mutagen
from mutagen import  flac,id3,oggvorbis
from mutagen.flac import FLAC
from mutagen.id3 import ID3, TIT2,TIT3,TALB,TCOP,USER,TRDA,TXXX,TCON,TOPE

from mutagen.oggvorbis import OggVorbis


def get_date(f_filename):
    c = time.localtime(os.lstat(f_filename)[stat.ST_MTIME])
    return str(c[0]) + "-" + str(c[1]) + "-" + str(c[2])


def build_dic(f_infos,f_text, f_date):
    l_dic = {}
    l_dic["SWAC_SPEAK_NAME"] = f_infos.locutor_name
    l_dic["SWAC_SPEAK_LIV_TOWN"] = f_infos.locutor_living_town
    l_dic["SWAC_SPEAK_LIV_COUNTRY"] = f_infos.locutor_living_country
    if f_infos.locutor_gender == 0:
        l_dic["SWAC_SPEAK_GENDER"] = u"M"
    else:
        l_dic["SWAC_SPEAK_GENDER"] = u"F"
    l_dic["SWAC_SPEAK_BIRTH_YEAR"] = f_infos.locutor_year_of_birth
    l_dic["SWAC_SPEAK_LANG"] = f_infos.locutor_native_langage
    l_dic["SWAC_SPEAK_LANG_COUNTRY"] = f_infos.locutor_linguistic_region
    l_dic["SWAC_COLL_LICENSE"] = f_infos.locutor_license
    l_dic["SWAC_COLL_COPYRIGHT"] = f_infos.locutor_copyright
    l_dic["SWAC_LANG"] = f_infos.list_langage
    l_dic["SWAC_COLL_NAME"] = f_infos.list_name
    l_dic["SWAC_TECH_DATE"] = f_date
    l_dic["SWAC_TECH_SOFT"] = "Yazikrecorder"
    if f_infos.name_style == 0:
        l_dic["YAZIK_NAME_STYLE"] = "Yazik"  
    elif f_infos.name_style == 1:
        l_dic["YAZIK_NAME_STYLE"] = "Shtooka"
    elif f_infos.name_style == 2:
        l_dic["YAZIK_NAME_STYLE"] = "Wikidictionnary"
    else:
        l_dic["YAZIK_NAME_STYLE"] = "Other"
    l_dic["SWAC_TEXT"] = f_text
    return l_dic
    
    
def tag_flac_file(f_filename,f_dic):
#    try:
    f = FLAC(f_filename)
    for k in f_dic.keys():
        f[k] = f_dic[k]
    f["TITLE"] = f_dic["SWAC_COLL_NAME"] + " - " + f_dic["SWAC_TEXT"]
    f["ARTIST"] = f_dic["SWAC_SPEAK_NAME"]
    f["GENRE"] = u"Speech"
    f["DATE"] = f_dic["SWAC_TECH_DATE"]
    f["LICENSE"] = f_dic["SWAC_COLL_LICENSE"]
    f["COPYRIGHT"] = f_dic["SWAC_COLL_COPYRIGHT"]
    f["ALBUM"] = f_dic["SWAC_COLL_NAME"]
    f.save()
#    except:
#        print "problem with tagging the following flac file: " , f_file

def tag_ogg_file(f_filename,f_dic):
    f = OggVorbis(f_filename)
    for k in f_dic.keys():
        f[k] = f_dic[k]
    f["TITLE"] = f_dic["SWAC_COLL_NAME"] + " - " + f_dic["SWAC_TEXT"]
    f["ARTIST"] = f_dic["SWAC_SPEAK_NAME"]
    f["GENRE"] = u"Speech"
    f["DATE"] = f_dic["SWAC_TECH_DATE"]
    f["LICENSE"] = f_dic["SWAC_COLL_LICENSE"]
    f["COPYRIGHT"] = f_dic["SWAC_COLL_COPYRIGHT"]
    f["ALBUM"] = f_dic["SWAC_COLL_NAME"]
    f.save()

def tag_mp3_file(f_filename,f_dic):
    try:
        f = mutagen.id3.ID3(f_filename)
    except mutagen.id3.ID3NoHeaderError:
        print "No ID3 tags: adding ..."
        f = mutagen.id3.ID3()
    except Exception, err:
        print str(err)
    #f = ID3(f_filename)
    for k in f_dic.keys():
        f.add(TXXX(encoding=3, desc=k,text=f_dic[k]))
    f.add(TIT2(encoding=3, text= f_dic["SWAC_COLL_NAME"] + " - " + f_dic["SWAC_TEXT"]))
    f.add(TCON(encoding=3, text=u"Speech"))
    f.add(TALB(encoding=3, text=f_dic["SWAC_COLL_NAME"]))
    f.add(TRDA(encoding=3, text=f_dic["SWAC_TECH_DATE"]))
    f.add(TXXX(encoding=3, desc="TOPE",text=f_dic["SWAC_SPEAK_NAME"]))
    f.add(TCOP(encoding=3, text=f_dic["SWAC_COLL_COPYRIGHT"]))
    f.add(USER(encoding=3, text=f_dic["SWAC_COLL_LICENSE"]))
    f.save(f_filename)


def read_ogg_tags(f_filename):
    f = OggVorbis(f_filename)
    for i in f.keys():
        l_dic[i] = f[i]
    return l_dic

def read_flac_tags(f_filename):
    # try 
    f = FLAC(f_filename)
    for i in f.keys():
        l_dic[i] = f[i]
    return l_dic

def reaf_mp3_tags(f_filename):
    pass

def build_custom_tags_dic():
    pass

def write_ogg_tags(f_dic,f_filename):
    pass

def write_flag_tags(f_dic,f_filename):
    pass

def write_mp3_tags(f_dic,f_filename):
    pass
    