#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

import sys, os

class Country:
    """
    this class is responsible of the correctness of the naming of
    the country names according to the ISO-3166 norm.
    http://www.iso.org/iso/en/prods-services/iso3166ma/02iso-3166-code-lists/index.html
    the file has been changed to unicode UTF8 coding and the DOS endlines
    changed to UNIX style.
    See the conv.sh script.
    """
    comprehensive_list = []
    real_iso_dic = {}
    iso_real_dic = {}
        
    def __init__(self):
        l_dir = os.path.dirname(__file__)
        self.l_file_name = os.path.join (l_dir, "list-en1-semic.utf8")
        # DEBUG:
        #print self.l_file_name
        # end debug



    def get_real_name(self,f_iso_name):
        if self.exist_iso_name(f_iso_name):
            return self.iso_real_dic[unicode(f_iso_name,"utf-8")]
        else:
            return None

    def get_iso_name(self,f_real_name):
        if self.exist_real_name(f_real_name):
            return self.real_iso_dic[unicode(f_real_name,"utf-8")]
        else:
            return None

    def exist_iso_name(self,f_iso_name):
        if unicode(f_iso_name,"utf-8") in self.iso_real_dic.keys():
            return True
        else:
            return False

    def exist_real_name(self,f_real_name):
        if unicode(f_real_name,"utf-8") in self.real_iso_dic.keys():
            return True
        else:
            return False

    def get_list(self):
        return self.comprehensive_list

    def main(self):
        #print "initialisation country"
        
        try:
            self.l_file = open(self.l_file_name,"r")
        except:
            print "Error: file list-en1-semic.utf8 doesn't exist"
            sys.exit(1)
        # debug
        #print ("country file OK")
        # first, pass the first two lines
        line = self.l_file.readline()
        line = self.l_file.readline()
        # next, read all lines and populate the dictionnaries
        while (1):
            line = self.l_file.readline()
            if line == "":
                break
            (l_name,l_code) = line.split(";")
            l_code = l_code.rstrip("\n\f\r")
            l_code = l_code.strip()
            l_name = l_name.strip()
            # DEBUG:
            # print l_name,"-",l_code
            self.real_iso_dic[unicode(l_name,"utf-8")] = unicode(l_code,"utf-8")
            self.iso_real_dic[unicode(l_code,"utf-8")] = unicode(l_name,"utf-8")
            self.comprehensive_list.append(unicode(l_code,"utf-8") + "\t" + unicode(l_name,"utf-8"))
        self.l_file.close()
        # DEBUG:
        # print self.comprehensive_list


if __name__ == "__main__":
    # testing the library ...
    my_country_table = Country()
    my_country_table.main()
    print my_country_table.real_iso_dic.keys()
    print my_country_table.iso_real_dic.keys()
    print my_country_table.get_real_name("AS")
    print my_country_table.get_real_name("TOTO")
    print my_country_table.get_iso_name("AMERICAN SAMOA")
    print my_country_table.get_iso_name("tortor")


