#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

import sys,os

class Langage:
    """
    this class is responsible for the correctness of the naming
    of langage according to the ISO-639 norm  .
    http://www.sil.org/iso639-3/download.asp
    """
    comprehensive_list = []
    real_iso_dic = {}
    iso_real_dic = {}
    
    def __init__(self):
        l_dir = os.path.dirname(__file__)
        self.l_file_name = os.path.join (l_dir, "iso-639-3_Name_Index_20070323.tab")
        # DEBUG:
        #print self.l_file_name
        # end debug
        #self.main()

    def get_real_name(self,f_iso_name):
        if self.exist_iso_name(f_iso_name):
            return self.iso_real_dic[unicode(f_iso_name,"utf-8")]
        else:
            return None

    def get_iso_name(self,f_real_name):
        if self.exist_real_name(f_real_name):
            return self.real_iso_dic[unicode(f_real_name,"utf-8")]
        else:
            return None

    def exist_iso_name(self,f_iso_name):
        if unicode(f_iso_name,"utf-8") in self.iso_real_dic.keys():
            return True
        else:
            return False

    def exist_real_name(self,f_real_name):
        if unicode(f_real_name,"utf-8") in self.real_iso_dic.keys():
            return True
        else:
            return False

    def get_list(self):
        return self.comprehensive_list

    def main(self):
        #print "initialisation langage"
        
        try:
            self.l_file = open(self.l_file_name,"r")
        except:
            print "Error: file  iso-639-3_Name_Index_20070323.tab doesn't exist"
            sys.exit(1)
        # debug
        #print ("langage file OK")
        self.comprehensive_list = []
        # first, pass the first  line
        line = self.l_file.readline()
        while (1):
            line = self.l_file.readline()
            if line == "":
                break
            (l_code,l_print_name,l_name) = line.split("\t")
            l_name = l_name.rstrip("\n\f\r")
            l_name = l_name.strip()
            l_code = l_code.strip()
            l_print_name = l_print_name.strip()
            # debug
            # print l_code,"-",l_print_name
            # end debug
            self.real_iso_dic[unicode(l_print_name,"utf-8")] = unicode(l_code,"utf-8")
            self.iso_real_dic[unicode(l_code,"utf-8")] = unicode(l_print_name,"utf-8")
            self.comprehensive_list.append(unicode(l_code,"utf-8") + "\t" + unicode(l_name,"utf-8"))
        self.l_file.close()

if __name__ == "__main__":
    # all stuff here are for testing purpose
    my_langage_base = Langage()
    my_langage_base.main()
    #print my_langage_base.real_iso_dic.keys()
    #print my_langage_base.iso_real_dic.keys()
    print my_langage_base.get_real_name("one")
    print my_langage_base.get_real_name("TOTO")
    print my_langage_base.get_iso_name("Central Bicolano")
    print my_langage_base.get_iso_name("tortor")
