#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

import sys,os
import binascii

import langage
from langage import langage
import country
from country import country

class Name:
    """
    this class is responsible of the naming of the audio files.
    Names can be of crc32 form with soime additionnal informations
    or pure textual form like the wikidictionnary format.
    """

    langage_class = None
    country_class = None

    def __init__(self):
        self.langage_class = langage.Langage()
        self.country_class = country.Country()

    def get(self,f_word):
        return f_word

class CRC32Name(Name):

     def get(self,f_word):
         t = '%x' % binascii.crc32(f_word)
         if t[0] == "-":
             t = t[1:]
         return t

class ShtookaName(Name):
    
    def get(self,f_word,f_langage,f_country):
        t = '%x' % binascii.crc32(f_word)
        if t[0] == "-":
            t = t[1:]
        t = f_langage + "-" + t
        return t

class WikiDictionnaryName(Name):

    def get(self,f_word,f_langage,f_country):
        # test of f_langage and f_country
        # if not OK than return None
        if self.langage_class.exist_iso_name(f_langage) and \
           self.country_class.exist_iso_name(f_country):
            l_string = f_word.replace(" ","_")
            return  f_langage + "-" + f_country + "-" + l_string
        else:
            return '%x' % binascii.crc32(f_word)

class YazikName(Name):

    def get(self,f_word,f_langage,f_country):
        # test of f_langage and f_country
        # if not OK than return None
        my_CRC32Nameclass = CRC32Name()
        #print "language: ", f_langage
        #print "country: ", f_country
        if self.langage_class.exist_iso_name(f_langage) and \
           self.country_class.exist_iso_name(f_country):
            return  f_langage + "-" + f_country + "-" +  my_CRC32Nameclass.get(f_word)
        else:
            # DEBUG:
            print "Error in the language or country names"
            return '%x' % binascii.crc32(f_word)

if __name__ == "__main__":
    # all this stuff to test the library
    my_langage_base = langage.Langage()
    my_country_table = country.Country()
    my_nameclass = Name()
    print my_nameclass.get("toto")
    my_CRC32Nameclass = CRC32Name()
    print my_CRC32Nameclass.get("toto")
    print my_CRC32Nameclass.get("la maison sur la colline est belle")
    my_WikiDictionnaryNameclass = WikiDictionnaryName()
    print my_langage_base.exist_iso_name("fra")
    print my_country_table.exist_iso_name("FR")
    print "ben alors !! " , my_WikiDictionnaryNameclass.get("la maison est très belle","fra","FR")
    my_YazikNameclass = YazikName()
    print "ben alors encore !! " , my_YazikNameclass.get("la maison est très belle","fra","FR")

