#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

import pygtk
# mandatory for installation with both pygtk1 and pygtk2
pygtk.require("2.0")
import gtk
import gtk.glade

import joko
from joko import AlsaDevices


class Parameters:
    """
     this class is responsible of saving, reading and setting the
     parameters of the yazik recorder
    """
    parameters_dic = {}
    rate_range = ["8000", "16000", "22500","44100","48000","88200","96000","192000"]
    depth_range = ["8","16","24","32"]
    
    
    
    def __init__(self):
        #self.parameters_dic = {}
        self.usine_parameters()
        self.get_audio()

    def usine_parameters(self):
        self.parameters_dic["interval"] = 0.05
        self.parameters_dic["threshold"] = -25.0
        self.parameters_dic["run_length"] = 2.0
        self.parameters_dic["pre_length"] = 0.3
        self.parameters_dic["leaky"] = True
        self.parameters_dic["saturation_level"] = -1.0
        self.parameters_dic["minimum_size"] = 0.15
        self.parameters_dic["vu_max_noise"] = -30.0
        self.parameters_dic["vu_bloc_length"] = 0.05
        self.parameters_dic["vu_peaklevel"] = -90.0
        self.parameters_dic["vu_decaylevel"] = -90.0
        #
        self.parameters_dic["vu_background"] = [1,1,1,65535]
        self.parameters_dic["vu_pre_length_color"] = [11398,56632,6337,65535]
        self.parameters_dic["vu_minimum_size_color"] = [33483,21729,22476,65535]
        self.parameters_dic["vu_signal_color"] = [5930,4513,61037,65535]
        self.parameters_dic["vu_font"] = "Sans 10"
        #
        self.parameters_dic["wv_color1"] = [1,2,3,65535]
        self.parameters_dic["wv_color2"] = [61483,60408,8541,65535]
        self.parameters_dic["wv_color3"] = [65535,65535,65535,65535]
        self.parameters_dic["wv_color4"] = [61985,3140,6648,65535]
        self.parameters_dic["wv_background"] = [5,6,7,65535]
        self.parameters_dic["wv_font"] = "Sans Bold 14"
        # audio devices
        self.parameters_dic["recording_device"] = "hw:0"
        self.parameters_dic["recording_rate"] = "44100"
        self.parameters_dic["recording_depth"] = 16
        self.parameters_dic["playback_device"] = "hw:0"
        
        
    def get_audio(self):
        self.record_dic = AlsaDevices.GetAlsaList("capture")
        self.playback_dic = AlsaDevices.GetAlsaList("playback")
        #print self.record_dic
        #print self.playback_dic
        
        
        
    
    def colors_int(self,f_colors):
         j = []
         for i in f_colors:
             j.append(int(i))
         return j


    def read_parameters(self,f_file):
        #print "reading the parameters file"
        #print f_file
        try:
            l_file = open(f_file,"r")
            # DEBUG:
            #print "OK: conf file read"
            # interpreting the conf file
            # name = value
            for l in l_file:
                if (l[0] == "#") or (l == "\n"):
                    continue
                # DEBUG:print l
                (un, deux) =  l.split("=")
                # DEBUG: 
                #print un, deux
                un = un.strip()
                deux = deux.strip()
                self.parameters_dic[un] = deux
            #print "parameters before interpreting", self.parameters_dic
            # now we must interpret the values
            self.parameters_dic["interval"] = float(self.parameters_dic["interval"])
            self.parameters_dic["threshold"] = float(self.parameters_dic["threshold"])
            self.parameters_dic["run_length"] = float(self.parameters_dic["run_length"])
            self.parameters_dic["pre_length"] = float(self.parameters_dic["pre_length"])
            if self.parameters_dic["leaky"] == "True":
                self.parameters_dic["leaky"] = True
            else:
                self.parameters_dic["leaky"] = False
            self.parameters_dic["saturation_level"] = float(self.parameters_dic["saturation_level"])
            #print "saturation après lecture",self.parameters_dic["saturation_level"]
            self.parameters_dic["minimum_size"] = float(self.parameters_dic["minimum_size"])
            self.parameters_dic["vu_max_noise"] = float(self.parameters_dic["vu_max_noise"])
            self.parameters_dic["vu_bloc_length"] = float(self.parameters_dic["vu_bloc_length"])
            self.parameters_dic["vu_peaklevel"] = float(self.parameters_dic["vu_peaklevel"])
            self.parameters_dic["vu_decaylevel"] = float(self.parameters_dic["vu_decaylevel"])
            self.parameters_dic["vu_background"] = self.colors_int(self.parameters_dic["vu_background"].split(","))
            self.parameters_dic["vu_pre_length_color"] = self.colors_int(self.parameters_dic["vu_pre_length_color"].split(","))
            self.parameters_dic["vu_minimum_size_color"] = self.colors_int(self.parameters_dic["vu_minimum_size_color"].split(","))
            self.parameters_dic["vu_signal_color"] = self.colors_int(self.parameters_dic["vu_signal_color"].split(","))
            self.parameters_dic["vu_font"] = self.parameters_dic["vu_font"]
            self.parameters_dic["wv_color1"] = self.colors_int(self.parameters_dic["wv_color1"].split(","))
            self.parameters_dic["wv_color2"] = self.colors_int(self.parameters_dic["wv_color2"].split(","))
            self.parameters_dic["wv_color3"] = self.colors_int(self.parameters_dic["wv_color3"].split(","))
            self.parameters_dic["wv_color4"] = self.colors_int(self.parameters_dic["wv_color4"].split(","))
            self.parameters_dic["wv_background"] =  self.colors_int(self.parameters_dic["wv_background"].split(","))
            self.parameters_dic["wv_font"] = self.parameters_dic["wv_font"]
#            self.parameters_dic["recording_device"] = "hw:0"
#            self.parameters_dic["recording_rate"] = "44100"
#            self.parameters_dic["playback_device"] = "hw:0"
            #print self.parameters_dic
        except:
            # DEBUG:
            print "EXEPTION: File not readable: usine parameters used"
            self.usine_parameters()

    def write_colors(self, f_file,f_colors):
        for i in range (3):
            f_file.write(str(f_colors[i]))
            f_file.write(",")
        f_file.write(str(f_colors[3]))


    def get_colors(self,f_color):
        l_color = f_color.get_color()
        l_alpha = f_color.get_alpha()
        return([l_color.red,l_color.green,l_color.blue,l_alpha])

    def save_parameters(self,f_file):
        try:
            l_file = open(f_file,"w")
            l_file.write("# configuration file for the yazik recorder \n")
            l_file.write("# every line beginning with a # is ignored, as are the blank lines \n")
            l_file.write("\n")
            l_file.write("# pipeline parameters: time esspressed in sec, levels in db \n# colors in rgba notation \n")
            l_file.write("interval = " + str(self.parameters_dic["interval"]) + "\n")
            l_file.write("threshold = " + str(self.parameters_dic["threshold"]) + "\n")
            l_file.write("run_length = " + str(self.parameters_dic["run_length"]) + "\n")
            l_file.write("pre_length = " + str(self.parameters_dic["pre_length"]) + "\n")
            l_file.write("leaky = " + str(self.parameters_dic["leaky"]) + "\n")
            l_file.write("saturation_level = " + str(self.parameters_dic["saturation_level"]) + "\n")
            l_file.write("minimum_size = " + str(self.parameters_dic["minimum_size"]) + "\n")
            l_file.write("\n")
            l_file.write("# naming sheme: either: Yazik,Shtooka,Wikidictionnary \n")
            l_file.write("\n")
            l_file.write("# vumeter parameters (some values will be computed from the values from  \n")
            l_file.write("# the pipeline) \n")
            l_file.write("vu_max_noise = " + str(self.parameters_dic["vu_max_noise"]) + "\n")
            l_file.write("vu_bloc_length = " + str(self.parameters_dic["vu_bloc_length"]) + "\n")
            l_file.write("vu_peaklevel = " + str(self.parameters_dic["vu_peaklevel"]) + "\n")
            l_file.write("vu_decaylevel = " + str(self.parameters_dic["vu_decaylevel"]) + "\n")
            l_file.write("# vumeter's colors \n")
            l_file.write("# vumeter background color \n")
            l_file.write("vu_background = ")
            self.write_colors(l_file,self.parameters_dic["vu_background"])
            l_file.write("\n")
            l_file.write("# vumeter pre_length_color \n")
            l_file.write("vu_pre_length_color = ")
            self.write_colors(l_file,self.parameters_dic["vu_pre_length_color"])
            l_file.write("\n")
            l_file.write("# vumeter  minimum_size_color \n")
            l_file.write("vu_minimum_size_color = ")
            self.write_colors(l_file,self.parameters_dic["vu_minimum_size_color"])
            l_file.write("\n")
            l_file.write("# vumeter signal_color \n")
            l_file.write("vu_signal_color = ")
            self.write_colors(l_file,self.parameters_dic["vu_signal_color"])
            l_file.write("\n")
            l_file.write("# Vumeter font\n")
            l_file.write("vu_font = " + self.parameters_dic["vu_font"])
            l_file.write("\n")#
            l_file.write("\n")
            l_file.write("# wordview parameters \n")
            l_file.write("# normal word text color \n")
            l_file.write("wv_color1 = ")
            self.write_colors(l_file,self.parameters_dic["wv_color1"])
            l_file.write("\n")
            l_file.write("# normal word backgroud color \n")
            l_file.write("wv_color2 = ")
            self.write_colors(l_file,self.parameters_dic["wv_color2"])
            l_file.write("\n")
            l_file.write("# highlighted word text color \n")
            l_file.write("wv_color3 = ")
            self.write_colors(l_file,self.parameters_dic["wv_color3"])
            l_file.write("\n")
            l_file.write("# highlighted word background color \n")
            l_file.write("wv_color4 = ")
            self.write_colors(l_file,self.parameters_dic["wv_color4"])
            l_file.write("\n")
            l_file.write("# widget background color (not used right now) \n")
            l_file.write("wv_background = ")
            self.write_colors(l_file,self.parameters_dic["wv_background"])
            l_file.write("\n")
            l_file.write("# Word view font \n")
            l_file.write("wv_font = " + self.parameters_dic["wv_font"])
            l_file.write("\n\n")
            # devices
            l_file.write("# Audio Devices \n")
            l_file.write("# Audio recording device \n")
            l_file.write("recording_device = " + self.parameters_dic["recording_device"])
            l_file.write("\n")
            l_file.write("# Audio rate recording \n")
            l_file.write("recording_rate = " + self.parameters_dic["recording_rate"])
            l_file.write("\n")
            l_file.write("# Audio playing device \n")
            l_file.write("playback_device = " + self.parameters_dic["playback_device"])
            l_file.write("\n")
            l_file.write("# Audio recording depth \n")
            l_file.write("recording_depth = " + self.parameters_dic["recording_depth"])
            l_file.close()
            #print self.parameters_dic
        except:
            print "not possible to create the parameters file"

    def get_widgets(self,f_xml):
        self.xml = f_xml
        self.widget_interval = f_xml.get_widget("spinbutton1")
        self.widget_threshold = f_xml.get_widget("spinbutton2")
        self.widget_run_length = f_xml.get_widget("spinbutton3")
        self.widget_pre_length = f_xml.get_widget("spinbutton4")
        self.widget_leaky = f_xml.get_widget("combobox1")
        self.widget_saturation_level = f_xml.get_widget("spinbutton5")
        self.widget_minimum_size = f_xml.get_widget("spinbutton6")
        self.widget_vu_max_noise = f_xml.get_widget("spinbutton7")
        self.widget_vu_bloc_length = f_xml.get_widget("spinbutton8")
        self.widget_vu_peaklevel = f_xml.get_widget("spinbutton9")
        self.widget_vu_decaylevel = f_xml.get_widget("spinbutton10")
        self.widget_vu_background = f_xml.get_widget("colorbutton1")
        self.widget_vu_pre_length_color = f_xml.get_widget("colorbutton2")
        self.widget_vu_minimum_size_color = f_xml.get_widget("colorbutton3")
        self.widget_vu_signal_color = f_xml.get_widget("colorbutton4")
        self.widget_vu_font = f_xml.get_widget("fontbutton1")
        self.widget_wv_color1 = f_xml.get_widget("colorbutton10")
        self.widget_wv_color2 = f_xml.get_widget("colorbutton9")
        self.widget_wv_color3 = f_xml.get_widget("colorbutton8")
        self.widget_wv_color4 = f_xml.get_widget("colorbutton7")
        self.widget_wv_background = f_xml.get_widget("colorbutton6")
        self.widget_wv_font = f_xml.get_widget("fontbutton2")
        self.widget_recording_device = f_xml.get_widget("combobox2")
        self.widget_recording_device.connect("changed",self.on_combobox2_changed)
        self.widget_recording_rate = f_xml.get_widget("combobox3")
        self.widget_playback_device = f_xml.get_widget("combobox4")
        self.widget_recording_depth = f_xml.get_widget("combobox5")

        
    def on_combobox2_changed(self,widget,data=None):
        model = self.widget_recording_rate.get_model()
        model.clear()
        model1 = self.widget_recording_depth.get_model()
        model1.clear()
        #print "changed!"
        y = self.get_active_text1(self.widget_recording_device).split(" ")[0]
        #print y
        # TODO: maybe, find out what really is the default device ...
        if y == "default":
            y = "hw:0"
        f_rate , depth_list = AlsaDevices.GetRecordingSampleRate(y)
        # TODO: better managment ...
        x = 0
        xx = 0
        if type(f_rate) == type(1):
            #print f_rate
            model.append([f_rate])  
            xx = 0          
        else:
            #print f_rate.high, f_rate.low
            for l in self.rate_range:
                if int(f_rate.low) <= int(l) <= int(f_rate.high) :
                    model.append([l])
                    #print "le rate", l
                    if l == "44100":
                        xx = x
                    x = x + 1
        self.widget_recording_rate.set_active(xx)
        # the depth
        #print depth_list
        x = 0
        xx = 0
        for d in depth_list:
            #print " d", d, 
            model1.append([d])
            if d == 16:
                xx = x
            x = x + 1
        self.widget_recording_depth.set_active(xx)    
        
        
            
            
        
    def get_active_text(self,combobox,f_default):
        active = combobox.get_active()
        if active < 0 :
            return "Yazik"
        elif active == 1:
            return "Shtooka"
        elif active == 2:
            return "Wikidictionnary"
        else:
            return f_default
    
    def get_active_text1(self,combobox):
        model = combobox.get_model()
        iter = combobox.get_active_iter()
        return model.get_value(iter, 0)
    
    def get_parameters(self):
        # pipeline parameters
        self.get_widgets(self.xml)
        self.parameters_dic = {}
        self.parameters_dic["interval"] = self.widget_interval.get_value()
        self.parameters_dic["threshold"] = self.widget_threshold.get_value()
        self.parameters_dic["run_length"] = self.widget_run_length.get_value()
        self.parameters_dic["pre_length"] = self.widget_pre_length.get_value()
        self.parameters_dic["leaky"] = self.get_active_text(self.widget_leaky,"True")
        self.parameters_dic["saturation_level"] = self.widget_saturation_level.get_value()
        self.parameters_dic["minimum_size"] = self.widget_minimum_size.get_value()
        self.parameters_dic["vu_max_noise"] = self.widget_vu_max_noise.get_value()
        self.parameters_dic["vu_bloc_length"] = self.widget_vu_bloc_length.get_value()
        self.parameters_dic["vu_peaklevel"] = self.widget_vu_peaklevel.get_value()
        self.parameters_dic["vu_decaylevel"] = self.widget_vu_decaylevel.get_value()
        self.parameters_dic["vu_background"] = self.get_colors(self.widget_vu_background)
        self.parameters_dic["vu_pre_length_color"] = self.get_colors(self.widget_vu_pre_length_color)
        self.parameters_dic["vu_minimum_size_color"] = self.get_colors(self.widget_vu_minimum_size_color)
        self.parameters_dic["vu_signal_color"] = self.get_colors(self.widget_vu_signal_color)
        self.parameters_dic["vu_font"] =  self.widget_vu_font.get_font_name()
        self.parameters_dic["wv_color1"] = self.get_colors(self.widget_wv_color1)
        self.parameters_dic["wv_color2"] = self.get_colors(self.widget_wv_color2)
        self.parameters_dic["wv_color3"] = self.get_colors(self.widget_wv_color3)
        self.parameters_dic["wv_color4"] = self.get_colors(self.widget_wv_color4)
        self.parameters_dic["wv_background"] = self.get_colors(self.widget_wv_background)
        self.parameters_dic["wv_font"] = self.widget_wv_font.get_font_name()
        self.parameters_dic["recording_device"] = self.get_active_text1(self.widget_recording_device).split(" ")[0]
        self.parameters_dic["recording_rate"] = self.get_active_text1(self.widget_recording_rate)
        self.parameters_dic["playback_device"] = self.get_active_text1(self.widget_playback_device).split(" ")[0]
        self.parameters_dic["recording_depth"] = self.get_active_text1(self.widget_recording_depth)
        #print "parameters", self.parameters_dic
        
    def set_parameters(self):
        self.widget_interval.set_value(self.parameters_dic["interval"])
        self.widget_threshold.set_value(self.parameters_dic["threshold"])
        self.widget_run_length.set_value(self.parameters_dic["run_length"])
        self.widget_pre_length.set_value(self.parameters_dic["pre_length"])
        if self.parameters_dic["leaky"]:
            self.widget_leaky.set_active(0)
        else:
            self.widget_leaky.set_active(1)
        self.widget_saturation_level.set_value(self.parameters_dic["saturation_level"])
        self.widget_minimum_size.set_value(self.parameters_dic["minimum_size"])
        self.widget_vu_max_noise.set_value(self.parameters_dic["vu_max_noise"])
        self.widget_vu_bloc_length.set_value(self.parameters_dic["vu_bloc_length"])
        self.widget_vu_peaklevel.set_value(self.parameters_dic["vu_peaklevel"])
        self.widget_vu_decaylevel.set_value(self.parameters_dic["vu_decaylevel"])
        self.widget_vu_background.set_color(gtk.gdk.Color(self.parameters_dic["vu_background"][0], \
                                                          self.parameters_dic["vu_background"][1], \
                                                          self.parameters_dic["vu_background"][2]))
        self.widget_vu_pre_length_color.set_color(gtk.gdk.Color(self.parameters_dic["vu_pre_length_color"][0], \
                                                          self.parameters_dic["vu_pre_length_color"][1], \
                                                          self.parameters_dic["vu_pre_length_color"][2]))
        self.widget_vu_minimum_size_color.set_color(gtk.gdk.Color(self.parameters_dic["vu_minimum_size_color"][0], \
                                                                  self.parameters_dic["vu_minimum_size_color"][1], \
                                                                  self.parameters_dic["vu_minimum_size_color"][2]))
        self.widget_vu_signal_color.set_color(gtk.gdk.Color(self.parameters_dic["vu_signal_color"][0], \
                                                                  self.parameters_dic["vu_signal_color"][1], \
                                                                  self.parameters_dic["vu_signal_color"][2]))
        self.widget_vu_font.set_font_name(self.parameters_dic["vu_font"])
        self.widget_wv_color1.set_color(gtk.gdk.Color(self.parameters_dic["wv_color1"][0], \
                                                                  self.parameters_dic["wv_color1"][1], \
                                                                  self.parameters_dic["wv_color1"][2]))
        self.widget_wv_color2.set_color(gtk.gdk.Color(self.parameters_dic["wv_color2"][0], \
                                                                  self.parameters_dic["wv_color2"][1], \
                                                                  self.parameters_dic["wv_color2"][2]))
        self.widget_wv_color3.set_color(gtk.gdk.Color(self.parameters_dic["wv_color3"][0], \
                                                                  self.parameters_dic["wv_color3"][1], \
                                                                  self.parameters_dic["wv_color3"][2]))
        self.widget_wv_color4.set_color(gtk.gdk.Color(self.parameters_dic["wv_color4"][0], \
                                                                  self.parameters_dic["wv_color4"][1], \
                                                                  self.parameters_dic["wv_color4"][2]))
        self.widget_wv_background.set_color(gtk.gdk.Color(self.parameters_dic["wv_background"][0], \
                                                                  self.parameters_dic["wv_background"][1], \
                                                                  self.parameters_dic["wv_background"][2]))
        self.widget_wv_font .set_font_name(self.parameters_dic["wv_font"])
        model1 = self.widget_recording_device.get_model()
        model1.clear()
        x = 0
        for k in self.record_dic.keys():
            y = k + "  " + self.record_dic[k]
            model1.append([y])            
            #print "recording device " ,self.parameters_dic["recording_device"], y
            if self.parameters_dic["recording_device"] in y:
                self.widget_recording_device.set_active(x)
            x = x + 1
        #
        model2 = self.widget_playback_device.get_model()
        model2.clear()
        x = 0
        for k in self.playback_dic.keys():
            y = k + "  " + self.playback_dic[k]
            model2.append([y])
            if self.parameters_dic["playback_device"] in y:
                self.widget_playback_device.set_active(x)
            x = x + 1
        
        
            
            
        

if __name__ == "__main__":
    mes_parametres = Parameters()
    xml_glade = gtk.glade.XML("recorder.glade")
    mes_parametres.read_parameters("recorder.conf")
    dialogue = xml_glade.get_widget("dialog1")
    mes_parametres.get_widgets(xml_glade)
    mes_parametres.set_parameters()
    fin = dialogue.run()
    #print "fin: " ,fin
    mes_parametres.get_parameters()
    mes_parametres.save_parameters("recorder.conf")
