#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################


import pygst
pygst.require("0.10")
import gst
import pygtk
# mandatory for installation with both pygtk1 and pygtk2
pygtk.require("2.0")
import gtk
import gtk.glade
import os, sys

import name
from name import name

import joko
from joko import AlsaDevices #, AudioPreview

import statusbar

def clamp(x, min, max):
    if x < min:
        return min
    elif x > max:
        return max
    return x


class Pipeline1:
    """
    The first pipeline which records the sounds from the microphone
    and generate a flac file which will be if correct transmitted
    to the second pipeline which will tranform it in a correct
    audio file of type mp3,ogg,wav
    maybe I can add the tagging at the end of the process ... 'll see
    """

    def __init__(self,f_statusbar,f_wordsview,f_graph, f_wordlist, f_recorded_words_list, f_target,f_parameters):
        self.my_statusbar = f_statusbar
        self.wordsview = f_wordsview
        self.recorded_words_list = f_recorded_words_list
        self.graph = f_graph
        self.wordlist = f_wordlist
        self.target = f_target
        self.parameters = f_parameters
        self.word_names = []
        self.saturation = False

    def read_parameters(self):
        # call the parameters part and get the paramters dic
        # here only for dev : will remove this part later: done!
        self.interval = long(self.parameters["interval"]     * 1000000000)
        #print "interval :",self.interval
        self.threshold = self.parameters["threshold"]
        self.run_length = long(self.parameters["run_length"] * 1000000000)
        #print "run_length :",self.run_length
        self.pre_length = long(self.parameters["pre_length"] * 1000000000)
        #print "pre_length", self.pre_length
        self.leaky = self.parameters["leaky"]
        self.saturation_level = self.parameters["saturation_level"]
        self.recording_device = self.parameters["recording_device"]
        self.recording_rate = self.parameters["recording_rate"]
        self.recording_depth = self.parameters["recording_depth"]
        self.playback_device = self.parameters["playback_device"]


    def createTargetFD (self) :
        self.targetName = "/dev/null"
        try:
            self.targetFD = os.open (self.targetName, os.O_WRONLY)
        except:
            print "Error: could'nt open the /dev/null device"
            sys.exit(1)
        #print "init target fd=%s -=> %s" % (self.targetFD, self.targetName)
        return self.targetFD

    def message_handler(self,l_bus,l_message):
        st = l_message.structure
        name =  st.get_name()
        #niveau = "below"
        if name == "cutter":
            if st['above']:
                self.saturation = False
                self.graph.set_saturation_off()
                self.changeTargetFile (self.target + self.word_names[self.number] + ".flac")
                self.graph.set_start_toggle(True)
                self.time_beginning = gst.TIME_ARGS(st['timestamp'])
            else:
                self.changeTargetFile () # reset to /dev/null
                self.graph.set_start_toggle(False)
                self.time_end = gst.TIME_ARGS(st['timestamp'])
                # calculate the length of the audio part
                # DEBUG: print "debut :", self.time_beginning, " fin :",self.time_end
                beg = float(self.time_beginning.split(":")[2])
                en  = float(self.time_end.split(":")[2])
                if (( en - beg) < 0.2) or self.saturation :
                    # DEBUG:
                    #print "sound too short or saturation occured"
                    self.graph.set_restart()
                else:
                    # reset the graph
                    self.graph.set_restart()
                    self.recorded_words_list.add_word(self.word_names[self.number] + ".flac",self.wordlist[self.number] )
                    self.wordsview.normal(self.wordlist[self.number])
                    if self.number < (self.total_words -1):
                        self.wordsview.highlighted(self.wordlist[self.number+ 1])
                        self.my_statusbar.update("audiofile number: " + str(self.number + 1 ) + " Total audiofiles: " + str(self.l))
                    self.number = self.number + 1
                    if self.number > self.total_words -1:
                        self.my_statusbar.update("audiofile number: " + str(self.number) + " Total audiofiles: " + str(self.l))
                        self.pipeline.set_state(gst.STATE_READY)

        if name == "level":
            if st["decay"][0] > self.saturation_level:
                #print "saturation:", " " ,st["decay"]
                self.graph.set_saturation_on()
                self.saturation = True
            self.graph.freeze_notify()
            decay = clamp(st['decay'][0], -90.0, 0.0)
            peak = clamp(st['peak'][0], -90.0, 0.0)
            # DEBUG if peak > decay:
            #    print "ERROR: peak bigger than decay!"
            self.graph.set_decay(decay)
            self.graph.set_peaklevel(peak)
            self.graph.queue_draw_area(0, 0, self.graph.allocation.width, self.graph.allocation.height)

        return True

    def forward(self):
        # one word forward
        self.OnStop()
        if self.number < (self.total_words -1):
#            self.wordsview.normal(self.wordlist[self.number])
#            self.wordsview.highlighted(self.wordlist[self.number+ 1])
            self.number = self.number + 1
            self.wordsview.forward(self.wordlist[self.number])
            #self.wordsview.highlighted(self.wordlist[self.number])
        self.OnPlay()

    def backward(self):
        # one word back
        self.OnStop()
        if self.number > 0:
#            self.wordsview.normal(self.wordlist[self.number])
            self.number = self.number - 1
            self.wordsview.backward(self.wordlist[self.number])
#            self.wordsview.highlighted(self.wordlist[self.number])
        self.OnPlay()

    def changeTargetFile (self,name="/dev/null") :
        # TODO: error management
        #print "change target -=>", name
        fd = os.open (name, os.O_WRONLY|os.O_CREAT)
        os.dup2 (fd, self.targetFD)
        os.close (fd) # ne pas oublier de fermer aprￃﾨs duplication
        self.targetName = name

    def affiche(self):
        print "*"

    def OnPlay(self):
        #print "Record"
        self.pipeline.set_state(gst.STATE_PLAYING)

    def OnStop(self):
        #print "Stop"
        self.pipeline.set_state(gst.STATE_READY)

    def pause(self):
        self.cutter.set_property("threshold-dB", -1)

    def record(self):
        self.cutter.set_property("threshold-dB", self.threshold)

    def halt(self):
        self.pipeline.set_state(gst.STATE_NULL)
        #del self.pipeline
        if hasattr(self, "targetFD"):
            #print "close target fd=%s -=> %s" % (self.targetFD, self.targetName)
            os.close(self.targetFD)
            del self.targetFD

    def set_parameters(self):
        pass
#        if hasattr(self, "targetFD"):
#            print "close target fd=%s -=> %s" % (self.targetFD, self.targetName)
#            os.close(self.targetFD)
#            del self.targetFD
#        self.set_pipeline()
#        self.OnPlay()
    
    def set_pipeline(self):
        self.pipeline = gst.Pipeline("mypipeline")
        self.source = gst.element_factory_make("alsasrc","audio")
        # here select the input source (see parameters)
        self.source.set_property("device",self.parameters["recording_device"])
        self.pipeline.add(self.source)
        
        # here, we add the caps filter to select the rate
        self.capsfilter = gst.element_factory_make("capsfilter")
        capsString = "audio/x-raw-int,rate=%s,depth=%s" % (self.parameters["recording_rate"],self.parameters["recording_depth"])
        caps = gst.caps_from_string(capsString)
        self.capsfilter.set_property("caps", caps)
        self.pipeline.add(self.capsfilter)
        #
        self.convert = gst.element_factory_make("audioconvert","audioconvert")
        self.pipeline.add(self.convert)
        self.level = gst.element_factory_make ("level", "level");
        self.level.set_property('interval', self.interval)
        self.pipeline.add(self.level)
        self.cutter = gst.element_factory_make ("cutter", "cutter");
        self.cutter.set_property("threshold-dB", self.threshold)
        self.cutter.set_property("run-length", self.run_length)
        self.cutter.set_property("pre-length", self.pre_length )
        self.cutter.set_property("leaky", self.leaky)
        self.pipeline.add(self.cutter)
        self.flac = gst.element_factory_make ("flacenc", "flac");
        self.pipeline.add(self.flac)
        self.sink = gst.element_factory_make("fdsink", "fdsink")
        # first send everything to  /dev/null
        self.sink.set_property("fd", self.createTargetFD())
        self.pipeline.add(self.sink)
        
        self.source.link(self.capsfilter)
        self.capsfilter.link(self.convert)
        self.convert.link(self.level)
        self.level.link(self.cutter)
        self.cutter.link(self.flac)
        self.flac.link(self.sink)
        self.bus = self.pipeline.get_bus()
        self.bus.add_watch (self.message_handler);
        self.bus.connect('message',self.message_handler )
    
    def main(self):
        self.name_class = name.Name()
        self.CRC32Name_class = name.CRC32Name()
        self.WikiDictionnaryName_class = name.WikiDictionnaryName()
        self.YazikName_class = name.YazikName()
        #my_name_class = name.CRC32Name()
        for names in self.wordlist:
            self.word_names.append(self.CRC32Name_class.get(names))
        
        if self.target[-1] != "/":
            self.target = self.target + "/"
        self.read_parameters()
        self.number = 0
        self.total_words = len(self.word_names) # don't forget to substract 1 ...
        self.set_pipeline()
        self.l = len(self.wordlist)
        if self.l > 0:
            self.wordsview.highlighted(self.wordlist[0])
