#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################################
#
#  Copyright (C) 2007 Eric Streit <Eric.Streit@wanadoo.fr>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################


import pygst
pygst.require("0.10")
import gst
import pygtk
# mandatory for installation with both pygtk1 and pygtk2
pygtk.require("2.0")
import gtk
import gtk.glade
import os, sys

#sys.path.append('/home/eric/workspace-yoxos/yazik2/src/lib/')

import svumeter
import wordsview
import recordedwordlist
import statusbar
import pipeline1
import parameters

# This variable contains the unique instance to the work window
TheRecorder = None

def LaunchRecorder (f_wordlist,f_target,f_instance):
    global TheRecorder
    if TheRecorder is None :
        TheRecorder = Recorder (f_wordlist,f_target,f_instance)
        TheRecorder.main()
    else:
        print "an instance of the recorder allready exists: not launching"
        # option 1: dialog box to complain that window is already open
        # option 2: reconfigure existing window and bring it to top
        # option 3: destroy old window then create a new one

class Recorder:
    """
    main class of the recorder: the gui
    f_wordlist is the list of words to be recorded (utf-8 coded ...)
    """

    def __init__(self,f_wordlist,f_target,f_instance):
        self.instance = f_instance
        self.wordlist = f_wordlist
        self.word_names = []
        self.target = f_target
        l_dir = os.path.dirname(__file__)
        l_file_name = os.path.join (l_dir, "recorder.glade")
        #print l_file_name
        self.xml_glade = gtk.glade.XML(l_file_name)
        dic={"on_window1_delete_event"            : self.on_window1_delete,
             "on_window1_destroy"                 : self.on_window1_destroy,
             "on_button1_clicked"                 : self.on_button1_clicked,
             "on_button2_clicked"                 : self.on_button2_clicked,
             "on_button3_clicked"                 : self.on_button3_clicked,
             "on_button4_clicked"                 : self.on_button4_clicked,
             "on_button5_clicked"                 : self.on_button5_clicked,
             "on_button6_clicked"                 : self.on_button6_clicked,
             "on_button7_clicked"                 : self.on_button7_clicked,
             "on_button8_clicked"                 : self.on_button8_clicked,
             "on_button9_clicked"                 : self.on_button9_clicked,
             "on_menuitem3_toggled"               : self.on_menuitem3_toggled,
             "on_gtk-preferences_activate_item"   : self.on_gtk_preferences_activate_item,
             "on_gtk-preferences_activate"        : self.on_gtk_preferences_activate_item,
             "on_imagemenuitem5_activate_item"    : self.on_imagemenuitem5_activate_item,
             "on_imagemenuitem5_activate"         : self.on_imagemenuitem5_activate_item,
             "on_imagemenuitem10_activate_item"   : self.on_imagemenuitem10_activate_item,
             "on_imagemenuitem10_activate"        : self.on_imagemenuitem10_activate_item #,
#             "on_combobox2_changed"               : self.on_combobox2_changed
             }
        self.xml_glade.signal_autoconnect(dic)
        #####################################################
        self.window1 = self.xml_glade.get_widget("window1")
        self.vpaned1 = self.xml_glade.get_widget("vpaned1")
        self.parameters_dialog = self.xml_glade.get_widget("dialog1")
        self.graph  = svumeter.SVUMeter()
        self.vpaned1.add2(self.graph)
        self.graph.show()


    def on_button1_clicked(self,widget,data=None):
        # DEBUG: record button
        #print "on button 1 clicked: record"
        self.button3.set_sensitive(False)
        self.button4.set_sensitive(False)
        self.gtk_preferences.set_sensitive(False)
        if self.wordlist != []:
            self.my_pipeline1.record()

    def on_button2_clicked(self,widget,data=None):
        # DEBUG: pause button
        #print "on button 2 clicked: pause"
        self.my_pipeline1.pause()
        self.button3.set_sensitive(True)
        self.button4.set_sensitive(True)
        self.gtk_preferences.set_sensitive(True)

    def on_button3_clicked(self,widget,data=None):
        # DEBUG: next
        #print "on button 3 clicked: next"
        self.my_pipeline1.forward()

    def on_button4_clicked(self,widget,data=None):
        # DEBUG: previous
        #print "on button 4 clicked: previous"
        self.my_pipeline1.backward()

    def on_button5_clicked(self,widget,data=None):
        # DEBUG: close
        #print "on button 5 clicked: close"
        self.window1.destroy()

    def on_button6_clicked(self,widget,data=None):
        # DEBUG: apply parameters
        #print "on button 6 clicked: apply parameters"
        self.my_parameters.get_parameters()
        self.graph.set_property("parameters",self.my_parameters.parameters_dic)
        self.graph.set_parameters()
        self.my_wordsview.set_parameters(self.my_parameters.parameters_dic)
        self.my_pipeline1.set_parameters()
        
    def on_button7_clicked(self,widget,data=None):
        # DEBUG: abort (parameters)
        #print "on button 7 clicked: abort"
        self.my_parameters.parameters_dic = self.my_saved_parameters
        self.my_parameters.set_parameters()
        self.parameters_dialog.hide()

    def on_button8_clicked(self,widget,data=None):
        # DEBUG: parameters record
        #print "on button 8 clicked: record the parameters"
        self.my_parameters.get_parameters()
        self.my_parameters.save_parameters(self.l_file_name)

    def on_button9_clicked(self,widget,data=None):
        # DEBUG:
        #print "on button 9 clicked"
        self.my_parameters.usine_parameters()

    def on_gtk_preferences_activate_item(self,widget,data=None):
        #self.my_pipeline1.halt()
        self.my_parameters.get_widgets(self.xml_glade)
        self.my_parameters.set_parameters()
        self.parameters_dialog.run()
        self.parameters_dialog.hide()
        #self.my_pipeline1.set_pipeline()

    def on_menuitem3_toggled(self,widget,data=None):
        #print "on menu_item3_toggled"
        x = self.word_per_line.get_active()
        self.my_wordsview.display_view(x)
        
        

    def on_imagemenuitem5_activate_item(self,widget,data=None):
        #print " on_imagemenuitem5_activate_item"
        self.window1.destroy()

    def on_imagemenuitem10_activate_item(self,widget,data=None):
        #print " on_imagemenuitem10_activate_item"
        self.dialogue = self.xml_glade.get_widget("aboutdialog1")
        self.dialogue.run()

    

    def on_window1_delete(self,widget,event,data=None):
        return False

    def  on_window1_destroy(self,widget,data=None):
        #print"it's good .."
        self.my_pipeline1.halt()
        print "updating the word dic"
        self.instance.to_be_updated = True
        self.instance.update_word_dic()
        self.instance.my_infos.save()
        global TheRecorder
        TheRecorder = None

    def main(self):
        # buttons widgets
        self.button1 = self.xml_glade.get_widget("button1")
        self.button2 = self.xml_glade.get_widget("button2")
        self.button3 = self.xml_glade.get_widget("button3")
        self.button4 = self.xml_glade.get_widget("button4")
        self.gtk_preferences = self.xml_glade.get_widget("gtk-preferences")
        self.word_per_line = self.xml_glade.get_widget("menuitem3")
        self.one_word_per_line = False
        
        self.my_parameters = parameters.Parameters()
        home_dir = os.path.expanduser("~")
        self.l_file_name = os.path.join(home_dir,".yazik/recorder.conf")
        # DEBUG:
        #print "1 " ,l_file_name
        self.my_parameters.read_parameters(self.l_file_name)
        self.my_saved_parameters = self.my_parameters.parameters_dic
        #print self.my_saved_parameters
        # we have now to set the parameters to the pipeline, the wordview widget and the graph widget
        self.graph.set_property("parameters",self.my_parameters.parameters_dic)
        self.graph.set_parameters()
        # DEBUG:
        #print self.word_names
        self.my_wordsview = wordsview.Wordsview(self.xml_glade.get_widget("textview1"),self.wordlist,self.my_parameters.parameters_dic)
        self.my_wordsview.main()
        self.my_recorded_words_list =  recordedwordlist.Recordedwordlist(self.xml_glade.get_widget("treeview1"))
        # DEBUG: self.my_recorded_words_list.add_word("toto")
        # DEBUG: self.my_recorded_words_list.set_name("Yazik form")
        self.my_statusbar = statusbar.Statusbar(self.xml_glade.get_widget("statusbar1"))
        # DEBUG self.my_status_bar.update("premier fichier enregistré")
        self.my_pipeline1 = pipeline1.Pipeline1(self.my_statusbar,self.my_wordsview,self.graph,self.wordlist,self.my_recorded_words_list, self.target,self.my_parameters.parameters_dic)
        self.my_pipeline1.main()
        self.my_pipeline1.pause() # pause put the threshold very high , so the recording cannot begin, but we can see the levels
        self.my_pipeline1.OnPlay()
        #gtk.main()


if __name__ == "__main__":
    my_recorder = Recorder(["un","deux","trois","la maison sur la colline","encore des choses à faire!","pouvez-vous me dire où se trouve l'aéroport",\
                            "comment s'appelle t-elle?", "voulez-vous du fromage"],"home/eric",True)
    my_recorder.main()
    gtk.main()
